#!/usr/bin/env node
'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _child_process = require('child_process');

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _fs = require('fs');

var _hostile = require('hostile');

var _hostile2 = _interopRequireDefault(_hostile);

var _os = require('os');

var _path = require('path');

var _updateNotifier = require('update-notifier');

var _updateNotifier2 = _interopRequireDefault(_updateNotifier);

var _package = require('../package.json');

var _package2 = _interopRequireDefault(_package);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

(0, _updateNotifier2.default)({ pkg: _package2.default }).notify();

// load existing config if any

var configFilePath = (0, _path.join)((0, _os.homedir)(), 'ffocus.json');
var config = void 0;
try {
  config = require(configFilePath);
} catch (e) {
  config = {};
}
var presets = config.presets || {};

// register cleanups

process.on('SIGINT', cleanup);
process.on('uncaughtException', cleanup);

// program

log('Welcome to ffocus 👋');

_commander2.default.usage('[minutes] [host1 host2 host3 ...]').option('-a, --add <host>', 'adds a new host').option('-r, --remove <host>', 'removes an existing host').option('-t, --duration <minutes>', 'sets the duration for a preset').option('-p, --preset <name>', 'a preset').option('-l, --list', 'lists all the presets').version(_package2.default.version);

_commander2.default.parse(process.argv);

// list presets

if (_commander2.default.list) {
  var availablePresets = Object.keys(presets);
  if (!availablePresets.length) {
    log('📚  No presets found.');
  } else {
    log('📚  Available presets:');
    Object.keys(presets).forEach(function (preset) {
      log('\uD83D\uDCDA  "' + preset + '" runs every ' + presets[preset].minutes + ' minutes');
      log('\uD83D\uDCDA  \tblocked sites:');
      presets[preset].sites.forEach(function (site) {
        return log('\uD83D\uDCDA  \t\t' + site);
      });
    });
  }
  process.exit(0);
}

var minutes = Number(_commander2.default.args[0]) || null;
var sites = _commander2.default.args.length > 1 ? _commander2.default.args.slice(1) : null;

// parse action (add, remove or duration)

var action = ['add', 'remove', 'duration'].reduce(function (acc, flag) {
  var action = _commander2.default[flag];
  if (action) {
    acc = {
      name: flag,
      value: action
    };
  }
  return acc;
}, null);

if (action) {
  // actions must have a preset
  if (!_commander2.default.preset) {
    error('--' + action.name + ' should be paired with --preset <name>');
  }

  var dummyPreset = {
    sites: [],
    minutes: 60
  };

  var actions = {
    add: function add(name, preset, host) {
      if (preset.sites.indexOf(host) !== -1) {
        error('"' + name + '" already contains ' + host);
        return original;
      }
      log('Adding ' + host + ' to "' + name + '"');
      return _extends({}, preset, {
        sites: preset.sites.concat([host])
      });
    },
    remove: function remove(name, preset, host) {
      presetExists(name);
      log('Removing ' + host + ' from "' + name + '"');
      return _extends({}, preset, {
        sites: preset.sites.filter(function (h) {
          return h !== host;
        })
      });
    },
    duration: function duration(name, preset, minutes) {
      presetExists(name);
      log('Setting the duration for "' + name + '" to ' + minutes + ' minutes');
      var mins = Number(minutes);
      if (isNaN(mins)) {
        error('"minutes" must be a number');
      }
      return _extends({}, preset, {
        minutes: mins
      });
    }
  };

  var update = actions[action.name](_commander2.default.preset, presets[_commander2.default.preset] || dummyPreset, action.value);

  // remove preset if empty
  if (!update.sites.length) {
    var _presets = presets,
        removed = _presets[_commander2.default.preset],
        rest = _objectWithoutProperties(_presets, [_commander2.default.preset]);

    presets = rest;
  }

  presets = Object.assign({}, presets, update.sites.length ? _defineProperty({}, _commander2.default.preset, update) : {});

  config = Object.assign({}, config, { presets: presets });

  try {
    write(config);
  } catch (err) {
    error('An error occurred while saving the configuration: ' + err.message, true);
  }

  log('Configuration saved.');
  process.exit(0);
}

// run a preset or one-off ffocus section

var preset = void 0;
if (_commander2.default.preset) {
  // preset
  presetExists(_commander2.default.preset);
  preset = presets[_commander2.default.preset];
} else {
  // one off
  if (!minutes || !sites) {
    _commander2.default.help();
  }
  preset = {
    minutes: minutes,
    sites: sites
  };
}

run(preset);

// helpers

function run(_ref2) {
  var minutes = _ref2.minutes,
      sites = _ref2.sites;

  if (!sites.length) {
    log('No sites to block.');
    process.exit(0);
  }

  sites.forEach(function (site) {
    try {
      _hostile2.default.set('127.0.0.1', site);
      log('\uD83D\uDEAB  Blocking ' + site);
    } catch (err) {
      error(err.message, true);
    }
  });

  setTimeout(function () {
    log('\uD83D\uDC4A  You\'re doing great!');
    log('\uD83D\uDC4A  You\'ve been productive for ' + minutes / 2 + ' minutes');
  }, minutes / 2 * 6e4);

  setTimeout(function () {
    log('\uD83D\uDE4F  Congrats you managed to focus for ' + minutes + ' minutes!');
    cleanup();
  }, minutes * 6e4);

  log('\u2705  Now focus for ' + minutes + ' minutes.');
}

function write(data) {
  (0, _fs.writeFileSync)(configFilePath, JSON.stringify(data, null, 2), { flag: 'w' }, 'utf8');
}

function cleanup() {
  var allSites = Object.keys(presets).reduce(function (acc, preset) {
    acc = acc.concat(presets[preset].sites);
    return acc;
  }, []).concat(sites).filter(Boolean);

  allSites.forEach(function (site) {
    try {
      _hostile2.default.remove('127.0.0.1', site);
    } catch (err) {
      error(err.message, true);
    }
  });

  process.exit(0);
}

function presetExists(name) {
  if (!presets[name]) {
    error('preset "' + name + '" not found');
  }
}

function log(msg) {
  console.log('\uD83D\uDD0D  ' + msg);
}

function error(msg, sudo) {
  console.error('\u2620\uFE0F  ' + msg);
  sudo && console.error('☠️  ffocus needs sudo rights to run');
  process.exit(1);
}
